@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2003-2011 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -21,7 +21,7 @@ from mercurial import util
 
 from PyQt4 import QtCore, QtGui
 
-from hgviewlib.util import Curry
+from hgviewlib.util import Curry, tounicode
 from hgviewlib.qt4 import icon as geticon
 
 Qt = QtCore.Qt
@@ -110,7 +110,13 @@ class FindQuickBar(QuickBar):
         connect(self._actions['cancel'], SIGNAL('triggered()'), self.cancel)
 
     def find(self, *args):
+        '''Scan the repository metadata and search for occurence of the
+        text in tre entry.
+        :note: do not scan if no text was provided'''
         text = unicode(self.entry.text())
+        if not text: # do not strip() as user may want to find space sequences
+            self.emit(SIGNAL('showMessage'), 'Nothing to look for.', 1000)
+            return
         if text == self.currenttext:
             self.emit(SIGNAL('findnext'), text)
         else:
@@ -199,9 +205,9 @@ class FindInGraphlogQuickBar(FindQuickBar):
         for node in graph[idx:]:
             rev = node.rev
             ctx = self._model.repo.changectx(rev)
-            if text in unicode(ctx.description(), 'utf-8', 'replace'):
+            # XXX should be an re search with undecoded chars as '?'
+            if text in tounicode(ctx.description()):
                 yield rev, None
-            pos = 0
             files = ctx.files()
             if self._filter_files:
                 files = [x for x in files if x in self._filter_files]
@@ -272,11 +278,13 @@ class FindInGraphlogQuickBar(FindQuickBar):
         for next_find in self._findinlog_iter:
             if next_find is None: # not yet found, let's animate a bit the GUI
                 if (step % 20) == 0:
-                    self.emit(SIGNAL("showMessage"), 'Searching'+'.'*(step/20))
+                    self.emit(SIGNAL("showMessage"),
+                              'Searching'+'.'*(step/20),
+                              -1)
                 step += 1
                 QtCore.QTimer.singleShot(0, Curry(self.find_next_in_log, (step % 80)))
             else:
-                self.emit(SIGNAL("showMessage"), '')
+                self.emit(SIGNAL("showMessage"), '', -1)
                 self.setCancelEnabled(False)
 
                 rev, filename = next_find
@@ -291,7 +299,9 @@ class FindInGraphlogQuickBar(FindQuickBar):
                         self._findinfile_iter = self._fileview.searchString(text)
                         self.on_findnext()
             return
-        self.emit(SIGNAL('showMessage'), 'No more matches found in repository', 2000)
+        self.emit(SIGNAL('showMessage'),
+                  'No more matches found in repository',
+                  2000)
         self.setCancelEnabled(False)
         self._findinlog_iter = None
 
@@ -311,19 +321,8 @@ class FindInGraphlogQuickBar(FindQuickBar):
                 self.emit(SIGNAL('showMessage'),
                           'Search string not found in current diff. '
                           'Hit "Find next" button to start searching '
-                          'in the repository', 2000)
+                          'in the repository',
+                          2000)
             else:
                 self.on_findnext()
 
-if __name__ == "__main__":
-    import sys
-    import hgviewlib.qt4 # to force importation of resource module w/ icons
-    app = QtGui.QApplication(sys.argv)
-    root = QtGui.QMainWindow()
-    w = QtGui.QFrame()
-    root.setCentralWidget(w)
-
-    qbar = QuickBar("test", "Ctrl+G", "toto", w)
-    root.show()
-    app.exec_()
-
